function [b] = bil_polygon(c1, c2)
% function [b] = bil_polygon(c1, c2)
%
% INPUTS:
% c1, c2     corner coordinates (listed in CCW order)
% OUTPUTS:
% b          billiard object containing lengths, area, etc
% barnett 8/13/06

b = [];
b.nc = numel(c1);          % number of corners
nc1 = circshift(c1, [0 -1]); nc2 = circshift(c2, [0 -1]);  % next corner
b.l = sqrt((c1-nc1).^2 + (c2-nc2).^2);
b.perim = sum(b.l);                          % l = side lengths
b.area = sum((c1-nc1).*(nc2+c2))/2;        % compute polygon area
b.th = atan2(nc2-c2, nc1-c1);              % start angs
b.ang = mod(pi - b.th + circshift(b.th, [0 1]), 2*pi);   % interior angles
b.bbox = [min(c1) max(c1) min(c2) max(c2)];        % bounding box
b.c1 = c1; b.c2 = c2;
b.type = 'p';

% end
