function [A, B] = corner_basis(na, c1, c2, th, x1, x2, k, nu, nx1, nx2)
%
% function [A {,B}] = corner_basis(na, c1, c2, th, x1, x2, k, nu ...
%                                    {, nx1, nx2})
% fill basis matrices with values (A) and optionally normal derivatives (B),
% for corner-adapted basis set. x1, etc are column vectors
% barnett 7/31/06

if nargout>1     % neumann wanted too
  [Rs, ths, ds1, ds2] = corner_basis_rth(na, c1, c2, th, x1, x2);
else
  [Rs, ths] = corner_basis_rth(na, c1, c2, th, x1, x2);
end
M = numel(x1);
N = numel(nu);
sintype = (nu<0);         % extract sin or cos type info
nu = abs(nu);             % correct the signs of nu
nus = repmat(nu, [M 1]);       % matrix of nu values
nusths = nus.*ths;
A = zeros(M,N);
if nargout>1         % do some set-up only needed for normal-derivs
  B = zeros(M,N);
  cosp = ds1.*repmat(nx1(:), [1 N]) + ds2.*repmat(nx2(:), [1 N]);
  sinp = ds1.*repmat(nx2(:), [1 N]) - ds2.*repmat(nx1(:), [1 N]);
  nusoRs = nus./Rs;   % slow?
end
for i=1:N                              % compute each column separately
  bess = besselj(abs(nu(i)), k*Rs(:,i));
  if (sintype(i))
    sins = sin(nusths(:,i));
    A(:,i) = sins.*bess;          % basis values
    if nargout>1
      coss = cos(nusths(:,i));
    end
  else                   % cos-type
    coss = cos(nusths(:,i));
    A(:,i) = coss.*bess;          % basis values
    if nargout>1
      sins = sin(nusths(:,i));
    end
  end
  if nargout>1
    % now use J'_nu(z) = J_{nu-1}(z) - (nu/z) J_nu(z)
    if (sintype(i))
      B(:,i) = cosp(:,i) .* sins .* (k*besselj(nu(i)-1, k*Rs(:,i)) - ...
                                bess.*nusoRs(:,i)) ...
          + sinp(:,i) .* coss .* bess .* nusoRs(:,i);
    else
      B(:,i) = cosp(:,i) .* coss .* (k*besselj(nu(i)-1, k*Rs(:,i)) - ...
                                bess.*nusoRs(:,i)) ...
          - sinp(:,i) .* sins .* bess .* nusoRs(:,i);
    end
  end
end
