function [Rs, ths, ds1, ds2] = corner_basis_rth(na, c1, c2, th, x1, x2)
% set up Rs (distances from basis corners)
% and ths (theta meas from start of each corner)
% arrays, given set of locations (x1,x2) for corner type basis
% location list is flattened to col vec, used as output format.
% Optionally return ds (unit vectors from basis corners to locations).

nc = numel(c1);
N = sum(na);
M = numel(x1);

dxc1 = repmat(x1(:), [1 nc]) - repmat(c1, [M 1]); % corner displacement matrix
dxc2 = repmat(x2(:), [1 nc]) - repmat(c2, [M 1]);
thc = atan2(dxc2, dxc1) - repmat(th, [M 1]);       % angles, j indexes corner
Rc = sqrt(dxc1.^2 + dxc2.^2);                      % dists, j indexes corner
dsc1 = dxc1./Rc; dsc2 = dxc2./Rc;
ths = zeros(M,N); Rs = zeros(M,N);
if nargout>2
  ds1 = zeros(M,N); ds2 = zeros(M,N);
end
col = 1;
for n=1:nc                                 % reindex: j now gives basis func
  ths(:,col:col+na(n)-1) = repmat(thc(:,n), [1 na(n)]);
  Rs(:,col:col+na(n)-1) = repmat(Rc(:,n), [1 na(n)]);
  if nargout>2           % want d unit vector from assoc corner to each loc
    ds1(:,col:col+na(n)-1) = repmat(dsc1(:,n), [1 na(n)]);
    ds2(:,col:col+na(n)-1) = repmat(dsc2(:,n), [1 na(n)]);
  end
  col = col + na(n);
end

%end

