% test fparamin tension-min searcher
%
% barnett 8/10/06

% dummy fun: return col vec list of values, will not accept a vector E input 
fun = @(E,Ej,cj,tj) sort(sqrt(cj.^2.*(E-Ej).^2 + 1e-2*(E-Ej).^3 + tj.^2))';
a = 1; b = 10;                 % interval to place minima within
n = 10;                        % how many minima to place there
Ej = sort(a + rand(1,n)*(b-a));
tj = rand(size(Ej))*1e-2;      % random (unsquared) tension minima
cj = 0.8+0.5*rand(size(Ej));   % slopes within certain limits

% plot dummy function...
Es = a:0.01:b;
f = zeros(numel(Ej), numel(Es));
for k=1:numel(Es), f(:,k) = fun(Es(k), Ej, cj, tj); end
figure; plot(Es, f, '.', 'markersize', 1); hold on; vline(Ej); 

% single test ================================================================
opts=[]; opts.ttyp=1e-2; opts.recur=3; cmin=0.6; cmax=1.5;
opts.maxparit=5; opts.select=1; opts.v=2; opts.maxdegen=100;
[E,t,C,info] = fparamin(a, b, cmin, cmax, opts, fun, Ej, cj, tj);
info.fevals
if numel(E)==numel(Ej)
  E - Ej
else
  E, Ej
end


% multiple tests ============================================================
wrong = 0; nt = 100;       % how many tests
opts.v = 0;
Ejw = []; tjw = []; cjw = [];   % arrays to collect wrong cases
for t = 1:nt                 % loop over trials --------------------------
  Ej = sort(a + rand(1,n)*(b-a));
  tj = rand(size(Ej))*1e-4;
  cj = 0.8+0.5*rand(size(Ej));
  [E,t,C,info] = fparamin(a, b, cmin, cmax, opts, fun, Ej, cj, tj);
  if numel(E)~=n
    maxerr = max(min(abs(repmat(Ej', [1 numel(E)])-repmat(E, [n 1]))));
    disp(sprintf('wrong # (%d): max dist from Ej = %g', numel(E), maxerr));
    Ejw = [Ejw; Ej]; tjw = [tjw; tj]; cjw = [cjw; cj]; 
    wrong=wrong+1;
  else
    disp(sprintf('ok #: max dist from Ej = %g', max(abs(E-Ej))));
  end
end
disp(sprintf('------------- %d wrong out of %d trials ----------', wrong, nt));

if 1
% show problem cases -----------------------------------------------------
opts.v = 2;
figure;
%for w=1:wrong               % loop over all problem cases
%  subplot(floor(sqrt(wrong)), ceil(sqrt(wrong)), w); 
for w=1                     % try a single problem case
  Ej = Ejw(w,:); cj = cjw(w,:); tj = tjw(w,:);  % recover problem cases
  f = zeros(numel(Ej), numel(Es));
  for k=1:numel(Es), f(:,k) = fun(Es(k), Ej, cj, tj); end
  plot(Es, f, '.', 'markersize', 1); hold on;
  [Ew,tw,Cw,info] = fparamin(a, b, cmin, cmax, opts, fun, Ej, cj, tj);
  info.fevals
  plot(Ew, tw, 'ko'); vline(Ej); title(sprintf('#=%d', numel(Ew))); drawnow;
end
end

if 0
% sandbox for diagnosing.....
figure;
i = 4;
am = info.as(i); bm = info.bs(i);
Es = am + (bm-am)*(0:0.005:1);
f = zeros(numel(Ej), numel(Es));
for k=1:numel(Es), f(:,k) = fun(Es(k), Ej, cj, tj); end
plot(Es, f, '.', 'markersize', 1); hold on;
f2 = @(E) fun(E,Ej,cj,tj).^2;
em = [am (am+bm)/2 bm];                     % set up for min_parabola
fm = [min(f2(am)) min(f2(em(2))) min(f2(bm))];
opts.Cmin = cmin^2;
[Ei,ti,ci,infoi] = min_parabola(f2, em, fm, 3, opts);      % choose recursion
vline(Ej(find(Ej<bm & Ej>am)));
end


if 0
Es = 7.22:0.0001:7.24;
f = zeros(numel(Ej), numel(E));
for k=1:numel(Es), f(:,k) = min(fun(Es(k), Ej, cj, tj)); end
plot(Es, f, '.', 'markersize', 1); hold on; vline(Ej);
end






% end
