% test split interval search code
%
% barnett 8/4/06

% dummy fun: return col vec list of values, will not accept a vector E input 
fun = @(E,Ej,cj,tj) sort(sqrt(cj.^2.*(E-Ej).^2 + 1e-2*(E-Ej).^3 + tj.^2))';
Ej = 1 + rand(1,10)*9;
tj = rand(size(Ej))*1e-2;
cj = 0.8+0.5*rand(size(Ej));
c = 0.8; d = 1.5;       % slope limits for search to use

% test the fun function...
E = 1:0.01:10;
f = zeros(numel(Ej), numel(E));
for k=1:numel(E), f(:,k) = fun(E(k), Ej, cj, tj); end
figure; plot(E, f, '.'); hold on; vline(Ej); 

% single test
opts = []; opts.ttyp = 1e-2;
[as,bs,ns,fcs] = split_interval(1,10,c,d, opts, fun, Ej, cj, tj);
line([as';bs'], kron([1;1],10-(1:numel(as))/10)); plot((as+bs)/2, fcs, 'o');
true_ns = zeros(size(as));        % diagnose
for i=1:numel(as)
  true_ns(i,1) = numel(find(as(i)<=Ej & Ej<=bs(i)));
end
if ~isempty(find(true_ns>1))
  disp(['not split: (' sprintf('%d',ns) ') should be (' ...
        sprintf('%d',true_ns) ')']);
end
if sum(true_ns)<numel(Ej)
  disp(sprintf('missed: only %d of %d eigvals included',...
               sum(true_ns), numel(Ej)));
end

if 0
  % use to locate minima: package way
  opts = optimset('fminbnd'); opts = optimset(opts, 'display', 'iter');
  Em = []; tm = [];
  for i=1:numel(as)
    [E,t] = fminbnd(@(E) min(fun(E,Ej,cj,tj))^2, as(i), bs(i), opts);
    % if (E~=as(i) & E~=bs(i))
    Em = [Em E]; tm = [tm sqrt(t)]; % using square of tension in parabolic min
  end
  
else
  opts = []; opts.v = 1; opts.xtol = 1e-4; opts.Cmin = 0.8^2;
  opts.maxparit = 5; %opts.select=0;
  Em = []; tm = [];
  for i=1:numel(as)
    fb2 = min(fun((as(i)+bs(i))/2,Ej,cj,tj))^2;
    [E,t] = min_parabola(@(E) min(fun(E,Ej,cj,tj))^2,as(i),bs(i),fb2,2,opts)
    Em = [Em E]; tm = [tm sqrt(t)]; % using square of tension in parabolic min
  end
end

line([as';bs'], kron([1;1],1e-3-(1:numel(as))*1e-4));
Em - sort(Ej)




% -------- test custom min parabola ----------------------
opts = []; opts.v = 1; opts.xtol = 1e-4; opts.Cmin = 0.8^2;
opts.maxparit = 5; %opts.select=0;
a = 9.6; c = 10; fb2 = min(fun((a+c)/2,Ej,cj,tj))^2;
E = a:(c-a)/300:c;
f = zeros(numel(Ej), numel(E));
for k=1:numel(E), f(:,k) = fun(E(k), Ej, cj, tj); end
figure; plot(E, min(f).^2, '-'); hold on; iEj=find(Ej>a & Ej<c);
if ~isempty(iEj), vline(Ej(iEj)); end; axis tight;
[E,t] = min_parabola(@(E) min(fun(E,Ej,cj,tj))^2, a, c, fb2, 2, opts);



% custom routine
Em = []; tm = []; opts = []; opts.xtol = 1e-4;
for i=1:numel(as)
  [E,t] = min_parabola(@(E) min(fun(E,Ej,cj,tj))^2, as(i),bs(i),fcs(i),opts);
  % if (E~=as(i) & E~=bs(i))
  Em = [Em E]; tm = [tm sqrt(t)];
  % end
end

% compare
sort(Em)
sort(Ej)

for i=1:9        % check it does sensible things... it does
  [as,bs,ns] = split_interval(i,i+1,c,d,1e-2, fun, Ej, cj, tj)
end


% multiple tests ===========================================================
wrong = 0; nt = 100;       % how many tests
Ejw = []; tjw = []; cjw = [];   % arrays to collect wrong cases
for t = 1:nt                 % loop over trials --------------------------
  Ej = 1 + rand(1,10)*9;         % random minima in [1,9]
  tj = rand(size(Ej))*1e-4;
  cj = 0.8+0.5*rand(size(Ej));
  %disp(sprintf('test %d:', t));
  opts = []; opts.ttyp = 1e-4;
  [as,bs,ns] = split_interval(1,10,c,d,opts, fun, Ej, cj, tj);
  true_ns = zeros(size(as));
  for i=1:numel(as)
    true_ns(i,1) = numel(find(as(i)<=Ej & Ej<=bs(i)));
  end
  if ~isempty(find(true_ns>1)) | sum(true_ns)<numel(Ej)
    if ~isempty(find(true_ns>1))
      disp([sprintf('test %d not split: (',t) sprintf('%d',ns) ') should be (' ...
            sprintf('%d',true_ns) ')']);
    end
    if sum(true_ns)<numel(Ej)
      disp(sprintf('test %d missed: only %d of %d eigvals included',...
                   t, sum(true_ns), numel(Ej)));
    end
    wrong=wrong+1;
    Ejw = [Ejw; Ej]; tjw = [tjw; tj]; cjw = [cjw; cj]; 
  end
end                         % --------------------------------------------
disp(sprintf('%d wrong out of %d tests.', wrong, nt));
% if all tj's < fc, then seems like only failure to split is close
% eigenvalues, ie works.

figure; E = 1:0.01:10;    % show problem cases
for w=1:wrong
  subplot(floor(sqrt(wrong)), ceil(sqrt(wrong)), w); 
  Ej = Ejw(w,:); cj = cjw(w,:); tj = tjw(w,:);  % recover problem cases
  f = zeros(numel(Ej), numel(E));
  for k=1:numel(E)
    f(:,k) = fun(E(k), Ej, cj, tj);
  end
  opts = []; opts.ttyp = 1e-2;
  [as,bs,ns] = split_interval(1,10,c,d,opts, fun, Ej, cj, tj);
  plot(E, f, '.'); hold on; vline(Ej);
  line([as';bs'], kron([1;1],10-(1:numel(as))/10));
end

%end

