function [i] = inside_bil(b, x1, x2, tol)
%function [i] = inside_bil(b, x1, x2 {, tol})
%
% returns true if (x1,x2) is inside the billiard object b.
% x1, x2 may be vectors, in which case i has size of x1.
% points within about tol of boundary will be counted as inside.
% For polygons done by counting winding number.
%
% barnett 8/13/06, influenced by Driscoll SC toolbox 2002,
% rad func added 10/30/06

if nargin<4
  tol = eps;
end

cx1 = x1(:); cx2 = x2(:);   % convert to columns

if b.type=='r' % -------------------------------- radial func ----------------
  i = ((cx1.^2+cx2.^2) <= b.f(atan2(cx2, cx1)).^2 + tol);
  
elseif b.type=='p' % ----------------------------- polygon -------------------
  % rect array of vectors from (x1,x2) to corners
  d1 = repmat(b.c1, [numel(x1) 1]) - repmat(cx1, [1 numel(b.c1)]);
  d2 = repmat(b.c2, [numel(x1) 1]) - repmat(cx2, [1 numel(b.c1)]);
  d = sqrt(d1.^2 + d2.^2);
  bad = d < eps;        % prevent dividing by zero (x hit a corner)
  d1(bad) = eps; d2(bad) = eps;
  th = atan2(d2, d1);           % angles of vectors from x's to corners
  % angles spanned by edges viewed from each x location, in [-pi,pi] range
  ang = mod(circshift(th, [0 -1]) - th + pi - tol, 2*pi) - pi + tol;

  winding = sum(ang,2) / (2*pi);   % winding numbers: 0 for outside, 1 inside
  i = reshape(abs(winding-1)<0.5, size(x1));    % 0.5 accounts for rounding err
  if any(winding<-0.5)
    disp('inside_bil: polygon corners not in CCW direction: winding<-0.5 !');
  end

else
  disp('inside_bil: bad billiard type!');
end

% end


