function [s] = param_spec(E, bcs, meth, bas, q)
%function [s] = param_spec(E, bcs, meth, bas, q)
%
% Returns tension eigenvalues function, given eigenvalue parameter E
% INPUTS:
% bcs      boundary condition info struct
% meth     computational method info struct (meth.v = verbosity)
% bas      basis info struct
% q        quadrature struct
%
% Needs option to compute & return corresp eigvecs
% barnett 8/12/06

verb = 0;
if isfield(meth, 'v')        % verbosity
  verb = opts.v;
end

k = sqrt(E);                % get wavenumber from eigenvalue

if (bcs.type=='d')          % DIRICHLET BCS
  if meth.nor=='i'         % interior values for norm
    % fill the A matrix with basis on bdry
    [A] = corner_basis(bas.na, bas.c1, bas.c2, bas.th, q.x1, q.x2, k, bas.nu);
    % choose norm as interior norm, via interior values
    B = corner_basis(bas.na, bas.c1, bas.c2, bas.th, q.y1, q.y2, k, bas.nu);
    B = diag(sqrt(q.wi))*B;
  else
    [A,B] = corner_basis(bas.na, bas.c1, bas.c2, bas.th, q.x1, q.x2, k, ...
                       bas.nu, q.nx1, q.nx2);
    B = diag(sqrt(q.w))*B/k;    % use bdry values to approx int norm, k-scaled
  end
  A = diag(sqrt(q.w))*A;
elseif bcs.type=='n'        % NEUMANN BCS
  [B,A] = corner_basis(bas.na, bas.c1, bas.c2, bas.th, q.x1, q.x2, k, ...
                       bas.nu, q.nx1, q.nx2);
  A = diag(sqrt(q.w))*A;
  if meth.nor=='i'         % interior values for norm
    B = corner_basis(bas.na, bas.c1, bas.c2, bas.th, q.y1, q.y2, k, bas.nu);
    B = diag(sqrt(q.wi))*B;
  else
    B = diag(sqrt(q.w))*B;    % use bdry values to approx interior norm
  end
end

% solve the GEP
if meth.gep=='s'                  % SVD
  s = svd(A);
elseif meth.gep=='g'              % Betcke
  s = gsvd(A,B);
elseif meth.gep=='v'              % reg GEVP
  [V,D] = eig(B'*B); mD=max(diag(D)); i=find(diag(D)>1e-14*mD);
  V=V(:,i); L=inv(sqrt(D(i,i)));
  if verb
    disp(sprintf('\t\tE=%g, B numerical rank=%d', k^2, numel(i)));
  end
  s = sqrt(eig(L*V'*A'*A*V*L)); % projects out num. Nul(B)
end

% end
